/* global $ */
$(document).ready(function() {
  init();
});

const cmSep = '||';
const cmSubSep = '|';

let blinktimer;
let blinkAltCol;
let blinkstate = false;

let isDoubleClick = false;
let prevArea = null;
let currentArea = null;
let actualQuestions = '';

const WVBSend = message => {
  window.ReactNativeWebView.postMessage(message);
};

// eslint-disable-next-line no-unused-vars
function consoleLog(message) {
  WVBSend('log' + cmSep + message);
}
// eslint-disable-next-line no-unused-vars
function consoleWarn(message) {
  WVBSend('warn' + cmSep + message);
}

// Handle message, called from app
// eslint-disable-next-line no-unused-vars
function handleMessage(mess) {
  const messArr = mess.split(cmSep);
  if (prevArea) {
    prevArea.attr('style', 'border-color: none');
  }
  routing(messArr);
}

function routing(messArr) {
  const paramArr = messArr[1].split(cmSubSep);

  switch (messArr[0]) {
    case 'color':
      return correctAnswer_action(paramArr);
    case 'blinkstop':
      return wrongAnswer_stop_action(paramArr);
    case 'blinkarea':
      return wrongAnswer_start_action(paramArr);
    case 'wronglabel':
      return showPermanentLabel_action(paramArr);
    case 'label':
      return showPermanentLabel_action(paramArr);
    case 'questions':
      actualQuestions = paramArr;
      return showQuestion_action();
    case 'tapType':
      return selectTapType_action(paramArr);
    case 'finished':
      return correctFlag_action();
  }
}

function correctAnswer_action(paramArr) {
  $('img#' + paramArr[0])
    .parent('.bsp')
    .attr('style', 'border-color:' + paramArr[1]);
}

function wrongAnswer_stop_action(paramArr) {
  const areaId = paramArr[0];
  stopBlink(areaId);
}

function wrongAnswer_start_action(paramArr) {
  const areaId = paramArr[0];
  const color = paramArr[1];
  startBlink(areaId, color);
}

function showPermanentLabel_action(paramArr) {
  showLabel(paramArr[0], paramArr[1], paramArr[2]);
}

function showQuestion_action() {
  $('img').each((idx, item) => {
    const id = $(item).attr('id');

    if (actualQuestions.indexOf(id) === -1) {
      $(item)
        .parent()
        .parent()
        .css('display', 'none');
    }
  });

  $('.container').css('opacity', '1');
  // Timeout needed
  setTimeout(function() {
    WVBSend('readyTwo');
  }, 100);
}

function selectTapType_action(paramArr) {
  isDoubleClick = paramArr[0] === 'LANG_XDOUBLETAP';
}

function correctFlag_action() {
  $('img').css('opacity', '1');
}

function startBlink(area, color) {
  if (blinktimer !== undefined) {
    return;
  }

  blinkAltCol = $('img#' + area)
    .parent('.bsp')
    .css('border-color');
  blinktimer = setInterval(function() {
    blink(area, color);
  }, 500);
}

function stopBlink(area) {
  // Do _not_ try to reset color here, that will overwrite the color: call!
  clearInterval(blinktimer);
  blinktimer = undefined;
  blinkstate = false;
}

function blink(area, color) {
  currentArea = area;
  const newcol = blinkstate ? color : blinkAltCol;
  $('img#' + area)
    .parent('.bsp')
    .attr('style', 'border-color:' + newcol);
  blinkstate = !blinkstate;
}

function init() {
  const container = $('.container');
  const items = container.find('.item');
  while (items.length) {
    const item = items.splice(Math.floor(Math.random() * items.length), 1)[0];
    container.append(item);
  }

  $('img').on('click', flagClick);

  setTimeout(function() {
    WVBSend('readyOne');
  }, 4500);
}

function mark(area) {
  if (prevArea) {
    prevArea.attr('style', 'border-color: none');
  }
  if (currentArea === area) {
    stopBlink();
  }

  prevArea = $('img#' + area).parent('.bsp');
  $('img#' + area)
    .parent('.bsp')
    .attr('style', 'border-color: #831e5b');
}

function flagClick(e) {
  e.preventDefault();
  if (isDoubleClick) {
    mark($(this).attr('id'));
  }
  WVBSend(['click', $(this).attr('id'), 'true'].join(cmSep));
}

function showLabel(id, value, fadeout) {
  fadeout = fadeout === undefined || fadeout === 'false' ? false : true;

  $('img#' + id)
    .parent()
    .prev()
    .text(value);
  if (fadeout) {
    setTimeout(function() {
      $('img#' + id)
        .parent()
        .prev()
        .text('');
    }, 2000);
  } else {
    $('img#' + id).attr('style', 'opacity: 0.1');
    $('img#' + id).css('pointer-events', 'none');
  }
}
