/* global $, window, screen */
$(document).ready(function() {
  init();
});

const svgNS = 'http://www.w3.org/2000/svg';

const cmSep = '||';
const cmSubSep = '|';

let isDoubleClick = false;
let prevArea = null;
let multiareaSelected = null;

let questions = [];
let answered = [];
let isFading = [];

const listeners = 'click';

const WVBSend = message => {
  window.ReactNativeWebView.postMessage(message);
};

// eslint-disable-next-line no-unused-vars
function consoleLog(message) {
  WVBSend('log' + cmSep + message);
}
// eslint-disable-next-line no-unused-vars
function consoleWarn(message) {
  WVBSend('warn' + cmSep + message);
}

// Handle message, called from app
// eslint-disable-next-line no-unused-vars
function handleMessage(mess) {
  const messArr = mess.split(cmSep);
  // if we have double click, remove coloring
  if (prevArea && messArr[0] !== 'orientation') {
    $(prevArea).attr('class', '');
    prevArea = null;
    deselectMulti();
  }
  routing(messArr);
}

function routing(messArr) {
  const paramArr = messArr[1].split(cmSubSep);

  switch (messArr[0]) {
    case 'color':
      return correctAnswer_action(paramArr);
    case 'blinkstop':
      return wrongAnswer_stop_action(paramArr);
    case 'blinkarea':
      return wrongAnswer_area_action(paramArr);
    case 'blinkcity':
      return wrongAnswer_city_action(paramArr);
    case 'label':
      return showPermanentLabel_action(paramArr, true);
    case 'truncatelabel':
      return showPermanentTruncatedLabel_action(paramArr, true);
    case 'wronglabel':
      return showPermanentLabel_action(paramArr, false);
    case 'orientation':
      return deviceRotated_action(paramArr);
    case 'questions':
      return darkenNonQuestionAreas_action(paramArr);
    case 'tapType':
      return selectTapType_action(paramArr);
    case 'locations':
      return addCustomLocations_action(paramArr);
    default:
      break;
  }
}

function correctAnswer_action(paramArr) {
  const areaId = paramArr[0];
  const color = paramArr[1];
  const isCity = paramArr[2];

  const area = findVisualArea(areaId);
  area.attr('class', '');
  showHideImage(area);
  addOpacity(area, color);
  $(area).css('fill', color);
  prevArea = null;
  if (isCity === 'true') {
    addBlackDot(area);
    area.css('opacity', 1);
  }
  answered.push(areaId);
}

function wrongAnswer_stop_action(paramArr) {
  const areaId = paramArr[0];
  stopBlink(areaId);
}

function wrongAnswer_city_action(paramArr) {
  const areaId = paramArr[0];
  const color = paramArr[1];

  startBlinkCity(areaId, color);
}

function wrongAnswer_area_action(paramArr) {
  const areaId = paramArr[0];
  const color = paramArr[1];
  startBlinkArea(areaId, color);
}

function showPermanentTruncatedLabel_action(paramArr, correctAnswer) {
  const areaId = paramArr[0];
  const rect = $('#' + areaId).find('rect');
  if (rect.length > 0) {
    paramArr[1] = truncateCXid($(rect[0]).attr('id'), paramArr[1]);
  }
  showPermanentLabel_action(paramArr, correctAnswer);
}

let hasC0Label = false;

function showPermanentLabel_action(paramArr, correctAnswer) {
  const areaId = paramArr[0];
  const localizedText = paramArr[1];
  const shouldFadeOut = paramArr[2];
  let answerColor = paramArr[3];
  const secondLabel = paramArr[4] || undefined;

  if (answerColor === 'detect') {
    answerColor = rgb2hex(findVisualArea(areaId).css('fill')).toLowerCase();
  }

  var bgColor = answerColor === undefined ? '#ffffff' : whiteTintColor(answerColor, areaId);

  hasC0Label = hasC0Rect(areaId);
  showLabel($('#' + areaId)[0], localizedText, shouldFadeOut, bgColor, correctAnswer, secondLabel);
}

function hasC0Rect(areaId) {
  const rects = $('#' + areaId).find('rect[id^="C0"]');
  return rects.length > 0;
}

function rgb2hex(value) {
  if (value.trim().substr(0, 4) !== 'rgb(') {
    return value;
  }
  const rgb = value
    .trim()
    .substr(4)
    .slice(0, -1)
    .split(',');
  return '#' + dualHex(rgb[0]) + dualHex(rgb[1]) + dualHex(rgb[2]);
}

function dualHex(dec) {
  const hex = parseInt(dec, 10).toString(16);
  return dec < 16 ? '0' + hex : hex;
}

function whiteTintColor(bgColor, areaId) {
  if (bgColor !== 'whitetint') {
    return bgColor;
  }
  const area = findVisualArea(areaId);
  const fill = $(area).css('fill');
  return ColorLuminance(fill, 0.1);
}

function truncateCXid(areaId, localizedText) {
  const pattern = /C([0-9]+)/;
  const matches = pattern.exec(areaId);

  if (matches == null || matches.length < 1) {
    return localizedText;
  }

  const letters = parseInt(matches[1], 10);
  if (letters < 1) {
    return localizedText;
  }
  return localizedText.substr(0, letters) + '.';
}

function deviceRotated_action(paramArr) {
  const orientation = paramArr[0];
  resizeMap(orientation);
}

function darkenNonQuestionAreas_action(paramArr) {
  questions = paramArr;
  $('svg')
    .find('g')
    .each(function(ix) {
      darkenAreaIfNotQuestion($(this).attr('id'));
      unmarkedHitareaHack($(this).attr('id'));
    });
  // OK, now we are really ready, unhide map
  showMap();
}

function showMap() {
  $('svg').css('opacity', '1');
  // Timeout needed
  setTimeout(function() {
    WVBSend('readyTwo');
  }, 100);
}

function selectTapType_action(paramArr) {
  const tapType = paramArr[0];
  isDoubleClick = tapType === 'LANG_XDOUBLETAP';
}

function stringContains(str, search) {
  if (!str) {
    return false;
  }
  return str.indexOf(search) !== -1;
}

function stringStartsWith(str, search) {
  if (!str) {
    return false;
  }
  return str.indexOf(search) === 0;
}

function darkenAreaIfNotQuestion(areaId) {
  if (!areaId) {
    return;
  }
  const parentId = $('#' + areaId)
    .parent()
    .attr('id');
  if (
    !stringStartsWith(areaId, 'MISC') &&
    !stringStartsWith(parentId, 'MISC') &&
    !stringContains(areaId, 'HIGHLIGHT') &&
    !stringContains(parentId, 'HIGHLIGHT') &&
    !stringContains(areaId, 'Isoleringsl') &&
    questions.indexOf(areaId) < 0
  ) {
    colorizeAreas(areaId);
  }
}

const FILL_COLOR = '#166c38';

function colorizeAreas(areaId) {
  // not if it is WATER
  if ($('#' + areaId).attr('id') === 'WATER') {
    return;
  }

  $('#' + areaId + ' polygon').css('fill', FILL_COLOR);
  $('#' + areaId + ' path').css('fill', FILL_COLOR);

  // rivers should be hidden, this needs to e explicit since maps are now reused with path fill none
  $('#' + areaId + " path[id^='RIVER']").css('stroke', FILL_COLOR);
  $('#' + areaId + " path[id^='RIVER']").remove();

  $('#' + areaId).css('pointer-events', 'none');

  $('#' + areaId + ' polyline').css('opacity', '0');
  $('#' + areaId + ' ellipse').each(hideNonQuestionCity);
  $('#' + areaId + ' circle').each(hideNonQuestionCity);
}

function hideNonQuestionCity(city) {
  if ($(this).attr('r') > 1 || $(this).attr('rx') > 1) {
    $(this).css('opacity', '0');
  }
}

function unmarkedHitareaHack(areaId) {
  // if svg g[id^="CITY_"] path has no fill, make it opacity 0
  // we have to do this in js, since there is no if in css
  // (and we must only set the opacity to 0 if fill is 'none')
  if (areaId && stringStartsWith(areaId, 'CITY_')) {
    const fill = $('#' + areaId + ' path').attr('fill');
    if (fill === 'none') {
      $('#' + areaId + ' path').css('opacity', '0');
    }
  }
}

function addCustomLocations_action(paramArr) {
  if (paramArr === undefined || paramArr === null) {
    return;
  }
  paramArr.map(loc => {
    const location = JSON.parse(loc.replace(/'/g, '"'));
    addCustomLocation(location);
  });
}

function dotTypeRadius(type) {
  switch (type) {
    case '1':
    case '1L':
      return 6;
    case '2':
      return 4;
    case '3':
      return 2;
    default:
      return 0;
  }
}

function makeCircle({ x, y, r }) {
  const circle = document.createElementNS(svgNS, 'circle');
  circle.setAttributeNS(null, 'cx', x);
  circle.setAttributeNS(null, 'cy', y);
  circle.setAttributeNS(null, 'r', r);
  return circle;
}

function makeRect({ x, y, width, height }) {
  const rect = document.createElementNS(svgNS, 'rect');
  rect.setAttributeNS(null, 'x', x);
  rect.setAttributeNS(null, 'y', y);
  rect.setAttributeNS(null, 'width', width);
  rect.setAttributeNS(null, 'height', height);
  return rect;
}

function addCustomLocation(location) {
  const { Id, X: Xs, Y: Ys, DotType } = location;
  const X = parseInt(Xs, 10);
  const Y = parseInt(Ys, 10);
  const radius = dotTypeRadius(DotType);

  const g = document.createElementNS(svgNS, 'g');
  const groot = document.createElementNS(svgNS, 'g');
  groot.setAttributeNS(null, 'id', `${Id}`);
  const hitarea = Math.max(radius * 2, 5);
  const circle1 = makeCircle({ x: X, y: Y, r: radius });
  circle1.setAttributeNS(null, 'id', `CITYR${radius}_${Id}`);
  const circle2 = makeCircle({ x: X, y: Y, r: hitarea });
  circle2.setAttributeNS(null, 'fill', 'none');

  const rect = makeRect({ x: X + 10, y: Y + 8 + radius, width: 20, height: 10 });
  rect.setAttributeNS(null, 'opacity', 0.0);

  if (DotType !== '1L') {
    rect.setAttributeNS(null, 'id', 'C0');
  }

  g.appendChild(rect);
  g.appendChild(circle1);
  g.appendChild(circle2);
  groot.appendChild(g);
  $('svg').append(groot);

  $(g).on(listeners, universalClick);
}

function findVisualArea(key) {
  const circle = $('#' + key + ' circle[id^="CITY"]');
  const circleID = $(circle).attr('id') === undefined ? '' : $(circle).attr('id');
  if (circleID.lastIndexOf('CITY', 0) === 0) {
    return $('#' + circleID);
  }
  const ellipse = $('#' + key + ' ellipse[id^="CITY"]');
  const ellipseID = $(ellipse).attr('id') === undefined ? '' : $(ellipse).attr('id');
  if (ellipseID.lastIndexOf('CITY', 0) === 0) {
    return $('#' + ellipseID);
  }
  const path = $('#' + key + ' path');
  if (path.length > 0) {
    return path;
  }
  const polygon = $('#' + key + ' polygon');
  if (polygon.length > 0) {
    return polygon;
  }
}

function addBlackDot(area) {
  if (
    $(area)
      .attr('id')
      .lastIndexOf('CITY', 0) === 0 &&
    $(area)
      .attr('id')
      .lastIndexOf('CITYR', 0) < 0
  ) {
    const dot = makeCircle({ x: $(area).attr('cx'), y: $(area).attr('cy'), r: 3 });
    dot.setAttributeNS(null, 'fill', '#000000');
    dot.setAttributeNS(null, 'id', 'blackdot');
    $(area)
      .parent()[0]
      .appendChild(dot);
  }
}

function showHideImage(area) {
  $(area)
    .parent()
    .find('[id^="SHOWAFTER"]')
    .css('opacity', 1);
  $(area)
    .parent()
    .find('[id^="SHOWBEFORE"]')
    .css('opacity', 0);
}

function addOpacity(area, color) {
  let ar = $(area)
    .parent()
    .find('[id^="SEMITRANSPARENT2"]');

  if (ar.length > 0) {
    if (color === '#ffffff') {
      ar.css('opacity', 0.5);
      ar.css('fill', color);
    } else {
      ar.css('opacity', 0.3);
      ar.css('fill', color);
    }
  } else if (
    $(area)
      .parent()
      .find('[id^="SEMITRANSPARENTVISIBLE"]').length > 0
  ) {
    ar = $(area)
      .parent()
      .find('[id^="SEMITRANSPARENTVISIBLE"]');
    ar.css('opacity', 0.1);
    ar.css('fill', color);
  } else if (
    $(area)
      .parent()
      .find('[id^="SEMITRANSPARENT"]').length > 0
  ) {
    ar = $(area)
      .parent()
      .find('[id^="SEMITRANSPARENT"]');
    ar.css('opacity', 0.1);
    ar.css('fill', color);
  }
}

// eslint-disable-next-line no-unused-vars
function hidePointer(t) {
  $(this).css('opacity', 0);
}

let blinktimer;
let blinkAltCol;

function startBlinkCity(area, color) {
  blinkAltCol = findVisualArea(area).css('fill');
  startBlink(area, color);
}

function startBlinkArea(area, color) {
  blinkAltCol = '#dddddd';
  startBlink(area, color);
}

function startBlink(area, color) {
  if (blinktimer !== undefined) {
    return;
  }
  blinktimer = setInterval(function() {
    blink(area, color);
  }, 500);
}

let blinkstate = false;

function stopBlink(area) {
  // Do _not_ try to reset color here, that will overwrite the color call!
  clearInterval(blinktimer);
  blinktimer = undefined;
  blinkstate = false;
}

function blink(area, color) {
  const newcol = blinkstate ? color : blinkAltCol;
  findVisualArea(area).css('fill', newcol);
  blinkstate = !blinkstate;
}

function makeSureThereIsAGContainer() {
  // <g> is most likely missing in the empty maps
  if ($('svg').find('g').length === 0) {
    const g = document.createElementNS(svgNS, 'g');
    $('svg').append(g);
  }
}

function init() {
  resizeMap();
  makeSureThereIsAGContainer();
  $('svg')
    .find('g')
    .each(initMapPart);
  setTimeout(function() {
    WVBSend('readyOne');
  }, 2500);
}

function resizeMap(orientation) {
  // Set size of map to as small as it can be while still covering viewport
  const svg = $('svg')[0];

  const viewBox = svg
    .getAttribute('viewBox')
    .trim()
    .split(' ');
  const origHeight = parseInt(viewBox[3], 10);
  const origWidth = parseInt(viewBox[2], 10);

  const origAspect = origWidth / origHeight;

  // Fun with webview differences: Our measured viewport size should care about portratit vs. landscape, but not care about page scaling.
  // On iOS, $(window).width() and $(window).height() does this.
  // On android, the only scalingindependent measure I could find was screen.width / screen.height, but that is only partially orientationindependent.
  // at start, they are correct, but when we tilt, they dont change, so we have to calculate viewport w/h from screen w/h and orientation.

  let wpHeight = $(window).height();
  let wpWidth = $(window).width();
  if ((navigator.userAgent || navigator.vendor).match(/Android/i)) {
    switch (orientation) {
      case 'LANDSCAPE':
        wpHeight = Math.min(screen.height, screen.width);
        wpWidth = Math.max(screen.height, screen.width);
        break;
      case 'PORTRAIT':
        wpHeight = Math.max(screen.height, screen.width);
        wpWidth = Math.min(screen.height, screen.width);
        break;
      default:
        // initial resize
        wpHeight = screen.height;
        wpWidth = screen.width;
        break;
    }
  }

  let newHeight = wpHeight;
  let newWidth = newHeight * origAspect;
  if (newWidth < wpWidth) {
    newWidth = wpWidth;
    newHeight = newWidth / origAspect;
  }

  svg.setAttribute('height', newHeight + 'px');
  svg.setAttribute('width', newWidth + 'px');

  const xOff = Math.max(0, (newWidth - wpWidth) / 2);
  const yOff = Math.max(0, (newHeight - wpHeight) / 2);

  window.scrollTo(xOff, yOff);
}

function initMapPart(mapPart) {
  // Adjust city circle radii, some maps (ex spanish_speaking_cap), has different radii
  // (cant style radii with css, so have to do it here)
  const thisId = $(this).attr('id');
  const cityCircleId = $(this)
    .find('circle[id^="CITY"]')
    .attr('id');
  if (thisId && cityCircleId && thisId.lastIndexOf('MISC', 0) < 0) {
    const radius = cityCircleId.substring(4, 5) === 'R' ? cityCircleId.substring(5, 6) : '6.86';
    $('circle#' + cityCircleId).attr('r', radius);
  }

  // Adjust any circles or ellipses with a radius <= 1. They should be styled as land (they are used for tiny pacific islands etc.)
  $(this)
    .find('circle')
    .each(fixTinyIslands);
  $(this)
    .find('ellipse')
    .each(fixTinyIslands);

  const path = $(this).find('path');
  const circle = $(this).find('circle');
  const polygon = $(this).find('polygon');
  const polyline = $(this).find('polyline');
  const ellipse = $(this).find('ellipse');
  const g = $(this).find('g');

  $(path).on(listeners, universalClick);
  $(circle).on(listeners, universalClick);
  $(polygon).on(listeners, universalClick);
  $(polyline).on(listeners, universalClick);
  $(ellipse).on(listeners, universalClick);
  $(g).on(listeners, universalClick);
}

function fixTinyIslands(candidate) {
  if ($(this).attr('r') <= 1 || $(this).attr('rx') <= 1) {
    $(this).attr('class', 'tinyisland');
  }
}

function openConfirm(area) {
  if (prevArea) {
    $(prevArea).attr('class', '');
    deselectMulti();
  }

  const selectArea = findVisualArea(area);

  if (questions.indexOf(area) >= 0) {
    selectMulti(area);
    prevArea = selectArea;
    selectArea.attr('class', 'selected');
  }
}

function selectMulti(area) {
  const circle = $('#' + area + ' circle[id^="CITY"]');
  if (circle.length > 1) {
    multiareaSelected = circle;
    circle.each(function() {
      $(this).attr('class', 'selected');
    });
  }
}

function deselectMulti() {
  if (multiareaSelected) {
    multiareaSelected.each(function() {
      $(this).attr('class', '');
    });
    multiareaSelected = null;
  }
}

function universalClick(e) {
  var cityID = $(this)
    .parent()
    .find('circle[id^="CITY"]')
    .attr('id');

  if (cityID === undefined) {
    cityID = $(this)
      .parent()
      .find('ellipse[id^="CITY"]')
      .attr('id');
    if (cityID === undefined) {
      cityID = '';
    }
  }

  if (
    isDoubleClick &&
    answered.indexOf(
      $(this)
        .parent()
        .attr('id'),
    ) < 0
  ) {
    openConfirm(
      $(this)
        .parent()
        .attr('id'),
    );
  }

  const cityClick = cityID.lastIndexOf('CITY', 0) === 0; // && cityID.lastIndexOf("CITYR", 0) < 0) ? "true" : "false";
  //	var cityClick = true; //handle if should be shown in the react part

  WVBSend(
    [
      'click',
      $(this)
        .parent()
        .attr('id'),
      cityClick,
    ].join(cmSep),
  );
}

function showLabel(obj, value, fadeout, bgColor, correctAnswer, secondLabel) {
  const secondPart = '_extra_info';
  const p = getCenterpoint(obj, correctAnswer);
  const attrId = obj.getAttribute('id');
  if (isFading.indexOf(attrId) !== -1) {
    return;
  }
  addText(p.x, p.y, value, attrId, getLabelColor(bgColor));
  setLineOpacity(obj, 1);

  if (secondLabel) {
    addText(p.x, p.y + 18, secondLabel, attrId + secondPart, getLabelColor(bgColor));
    setTimeout(function() {
      $('#RECT' + attrId + secondPart).fadeOut(500, function() {
        $('#RECT' + attrId + secondPart).remove();
      });
      $('#TEXT_' + attrId + secondPart).fadeOut(500, function() {
        $('#TEXT_' + attrId + secondPart).remove();
      });
    }, 750);
  }

  if (shouldFadeOutLabel(fadeout, correctAnswer) || hasC0Label) {
    if (isFading.indexOf(attrId) === -1) {
      isFading.push(attrId);
    }
    setTimeout(function() {
      $('#RECT' + attrId).fadeOut(1000, function() {
        $('#RECT' + attrId).remove();
        setLineOpacity(obj, 0);
      });
      $('#TEXT_' + attrId).fadeOut(1000, function() {
        $('#TEXT_' + attrId).remove();
        const index = isFading.indexOf(attrId);
        if (index !== -1) {
          isFading.splice(index);
        }
      });
    }, 1000);
  }
}

function getLabelColor(color) {
  switch (color) {
    case '#ffffff':
      return '#f4f4f4';
    case '#ffd970':
      return '#ffd05e';
    case '#e2b22d':
      return '#dba323';
    case '#bf4140':
      return '#c75352';
    default:
      return color;
  }
}

function shouldFadeOutLabel(fadeout, correctAnswer) {
  if (fadeout === undefined || fadeout === 'false' || fadeout === false) {
    return !correctAnswer;
  } else {
    return true;
  }
}

function getCenterpoint(g, correctAnswer) {
  const myRect = getAreaRectangle(g, correctAnswer);
  if (myRect != null) {
    return getAreaRectPosition(myRect);
  } else {
    const myCity = getCityCircle(g);
    if (myCity != null) {
      return getCityRectPosition(myCity);
    } else {
      return getStandardRectPosition(g);
    }
  }
  // return { x: 0, y: 0 };
}

function getCityCircle(g) {
  const n = $(g).find('circle[id^="CITY"]');
  return n[0];
}

function getAreaRectangle(g, correctAnswer) {
  let n = null;
  if (!correctAnswer) {
    n = $(g).find('rect[id^="UNANSWERED"]');
  }
  if (n == null || n.length === 0) {
    n = $(g).find('rect:not([id^="UNANSWERED"])');
  }

  let myRect = null;
  for (var i = 0; i < n.length; i++) {
    myRect = n[i];
  }
  return myRect;
}

function getAreaRectPosition(rect) {
  const x = rect.getBBox().x + rect.getBBox().width / 2;
  const y = rect.getBBox().y + rect.getBBox().height - 3;
  return { x: x, y: y };
}

function getCityRectPosition(rect) {
  const x = rect.getBBox().x + rect.getBBox().width / 2;
  const y = rect.getBBox().y + rect.getBBox().height / 2 + 20;
  return { x: x, y: y };
}

function getStandardRectPosition(rect) {
  const x = rect.getBBox().x + rect.getBBox().width / 2;
  const y = rect.getBBox().y + rect.getBBox().height / 2;
  return { x: x, y: y };
}

function addText(x, y, myText, myId, bgColor) {
  const xPadding = 6;
  const yPadding = 3;

  let myElem = document.getElementById('labelpoint');
  if (myElem === null) {
    myElem = document.getElementsByTagName('svg')[0];
  }
  // Hack warning: If bg is this specific color, make text white instead of black... Really, we should send classnames instead of bgcolor
  let textColor = bgColor === '#bf4140' ? '#ffffff' : '#000000';
  textColor = bgColor === '#c75352' ? '#ffffff' : textColor;

  const strokeColor = ColorLuminance(bgColor, -0.2);

  const newText = document.createElementNS(svgNS, 'text');
  newText.setAttributeNS(null, 'x', x);
  newText.setAttributeNS(null, 'y', y);
  newText.setAttributeNS(null, 'id', 'TEXT_' + myId);
  newText.setAttributeNS(null, 'class', 'svgLabelText');
  newText.setAttributeNS(null, 'fill', textColor);
  newText.appendChild(document.createTextNode(myText));
  myElem.appendChild(newText); // We must add it, or it has no size, which makes bg size wrong

  const newBox = makeRect({
    x: x - (newText.getBBox().width + xPadding) / 2,
    y: y - (newText.getBBox().height + yPadding) / 2 - yPadding,
    height: newText.getBBox().height + yPadding - 3,
    width: newText.getBBox().width + xPadding,
  });
  newBox.setAttributeNS(null, 'id', 'RECT' + myId);
  newBox.setAttributeNS(null, 'class', 'labelBkgrd');
  newBox.setAttributeNS(null, 'fill', bgColor);
  newBox.setAttributeNS(null, 'stroke', strokeColor);

  myElem.appendChild(newBox);

  myElem.appendChild(newText); // We must add it again since we cant use z-index to style elements in svg
}

function setLineOpacity(g, opacity) {
  const l = g.getElementsByTagName('line');

  for (let i = 0; i < l.length; ++i) {
    const temp = l[i].getAttribute('id');
    if (!temp || (temp && temp.indexOf('SHOWBEFORE') < 0)) {
      $(l[i]).css('opacity', opacity);
    }
  }
}

function ColorLuminance(hex, lum) {
  // validate hex string
  hex = String(hex).replace(/[^0-9a-f]/gi, '');
  if (hex.length < 6) {
    hex = hex[0] + hex[0] + hex[1] + hex[1] + hex[2] + hex[2];
  }
  lum = lum || 0;

  // convert to decimal and change luminosity
  let rgb = '#',
    c,
    i;

  for (i = 0; i < 3; i++) {
    c = parseInt(hex.substr(i * 2, 2), 16);
    c = Math.round(Math.min(Math.max(0, c + c * lum), 255)).toString(16);
    rgb += ('00' + c).substr(c.length);
  }

  return rgb;
}
